%	ANISO_CC_GRID Elliptic anisotropy parameters R and theta of gridded data.
%   [R, THETA_DEG]=ANISO_CC_GRID(XI,YI,ZI) returns the anisotropy ratio (R)
%   and the principal axes orientation angle (THETA_DEG) for a 2D random
%   field ZI measured on a recntagular grid [XI,YI]
%
%   The sample partial derivatives are evaluated using the method of
%   centered differences. 
%
%   THETA_DEG represents the angle between the x axis of the coordinate
%   system and the first (not necessarily the major) principal axis
%   R represents the ratio of the correlation length along the first
%   principal axis over the correlation lengh along the orthogonal 
%   principal axis. 
%
%   INPUT VARIABLES
%   XI:	Grid coordinates on X-axis
%   YI:	Grid coordinates on Y-axis
%   ZI:	Random Field on grid [XI,YI]
%
%	OUTPUT VARIABLES
%
%	R:          Anisotropy ratio
%	THETA_DEG:	Anisotropy orientiation angle (degrees)
%
%   COMMENTS:
%
%   The anisotropy ratio is defined as R = b1/b2, where b1 is the 
%   correlation length in the direction 1 (horizontal) and b2 is the 
%   correlation length in the direction 2 (vertical)  
%
%   EXAMPLE:
%
%   Generate and display a 2D random field with Matern covariance and
%   smoothness index NU=2, and then determine the anisotropy parameters
%   from the sample using ANISO_CC_GRID
%
%   [x,y,rf]=rf_gen(100,100,'Mate',0,0.5,4,6,20,2);
%   [R, THETA_DEG]=aniso_cc_grid(x,y,rf)
%
%   Code written by D.T. Hristopulos and A. Chorti
%   Last update: August 15, 2008


function [R, theta_deg]=aniso_cc_grid(xi, yi, zi)

stepx=xi(1, 2)-xi(1, 1);
stepy=yi(2, 1)-yi(1, 1);
[divZx, divZy]=gradient(zi, stepx, stepy);
Q11=mean(mean(divZx.*divZx, 1), 2);
Q22=mean(mean(divZy.*divZy, 1), 2);
Q12=mean(mean(divZx.*divZy, 1), 2);
Zdiag=Q22/Q11;
Zoff=Q12/Q11;
theta= atan2(2*Zoff,1-Zdiag)/2;
R=sqrt(1+((1-Zdiag)/(Zdiag-(1+Zdiag)*(cos(theta))^2)));
theta_deg=theta*180/pi;