%	ANISO_INTERP_SCATTER Anisotropy parameters R and THETA of a 2D random 
%   field sampled on a random grid (scattered data).  
%   [R, THETA_DEG]=ANISO_INTERP_SCATTER(X,Y,Z,LEN,METHOD,X0,XL,Y0,YL)
%   returns the anisotropy ratio (R)
%   and the principal axes orientation angle (THETA_DEG) for a 2D random
%   field Z sampled at scattered points [X, Y]. 
%
%   THETA_DEG represents the angle between the x axis of the coordinate
%   system and the first (not necessarily the major) principal axis
%   R represents the ratio of the correlation length along the first
%   principal axis over the correlation lengh along the orthogonal 
%   principal axis. 
%
%   First, the field is interpolated on a square grid of size LEN x LEN. 
%   The interpolation method can be one of the following choices: 
%   'LINEAR', 'CUBIC', 'BIHARMONIC SPLINE', 'NEAREST NEIGHBOR'. 
%
%   Second, the partial derivatives are evaluated using
%	centered differences. If the interpolation method is not specified, the
%   LINEAR interpolation method is used by default.
%
%	INPUT VARIABLES
%
%	X:      Sampling coordinates on X-axis (vector)
%	Y:      Sampling coordinates on Y-axis (vector)
%	Z:      Random Field on scattered points [X,Y] (vector)
%	LEN: 	Length of the interpolation grid. DEFAULT = length(Z)
%	METHOD: Interpolation method. Possible choices are: 
%           'linear' (DEFAULT)
%           'cubic'
%           'v4' (biharmonic spline)
%           'nearest'. 
%	X0:     X origin of the interpolation grid. DEFAULT = min(X)
%	Y0:     Y origin of  of the interpolation grid. DEFAULT = min(Y)
%	XL:     X end of the interpolation grid. DEFAULT = max(X)
%	YL:     Y end coordinate of the interpolation grid. DEFAULT = max(Y)
%
%	OUTPUT VARIABLES
%
%	R:          Anisotropy ratio
%	THETA_DEG:	Anisotropy orientiation angle (degrees)
%
%   COMMENTS:
%
%   The anisotropy ratio is defined as R = b1/b2, where b1 is the 
%   correlation length in the direction 1 (horizontal) and b2 is the 
%   correlation length in the direction 2 (vertical)  
%
%   EXAMPLE:
%
%   Generate and display a 2D random field with Gaussian covariance and
%   smoothness index NU=2 on a 256 x 256 grid.  Randomly select 4096 points 
%   from the grid to obtain a scattered sample. Determine the anisotropy 
%   parameters  from the sample using ANISO_INSTERP_SCATTER by interpola-
%   ting the scattered data on a 64 x 64 grid using linear interpolation. 
%
%   L=256; N=L^2;
%   Nr=4096; Lr=64; 
%   [x,y,rf]=rf_gen(L,L,'Gaus',0,0.5,8,12,20);
%   ind=randperm(N);
%   ind=ind(1:Nr);
%   Xtemp=reshape(x,N,1);
%   Ytemp=reshape(y,N,1);
%   RFtemp=reshape(rf,N,1);
%   x=Xtemp(ind);
%   y=Ytemp(ind);
%   rf=RFtemp(ind); clear Xtemp Ytemp RFTemp; 
%   [R, THETA_DEG]=aniso_interp_scatter(x,y,rf,Lr,'linear');
%
%   Code written by D.T. Hristopulos and A. Chorti
%   Last update: August 15, 2008

function [R, theta_deg]=aniso_interp_scatter(x, y, z, len, method, x0, xL, y0, yL)

if nargin==3
    len=length(z);
    method='linear';
    x0=min(x);
    xL=max(x);
    y0=min(y);
    yL=max(y);
elseif nargin==4
    method='linear';    
    x0=min(x);
    xL=max(x);
    y0=min(y);
    yL=max(y);
elseif nargin==5;
    x0=min(x);
    xL=max(x);
    y0=min(y);
    yL=max(y);
end
xn=linspace(x0,xL, round(sqrt(len)));
yn=linspace(y0,yL, round(sqrt(len)));
[xi,yi]=meshgrid(xn,yn); 
zi=griddata(x,y,z,xi,yi, method);

%   After the interpolation some locations have Nan values. These should be
%   excluded from the calculations
stepx=xi(1, 2)-xi(1, 1);
stepy=yi(2, 1)-yi(1, 1);
[divZx, divZy]=gradient(zi, stepx, stepy);
%	Remove Nan values
i_x=find(isnan(divZx));
c_x=length(i_x);
i_y=find(isnan(divZy));
c_y=length(i_y);
divZx(i_x)=0;
divZy(i_y)=0;

Q11=mean(mean(divZx.*divZx, 1), 2);
Q22=mean(mean(divZy.*divZy, 1), 2);
Q12=mean(mean(divZx.*divZy, 1), 2);

Zdiag=Q22/Q11;
Zoff=Q12/Q11;
theta=atan(2*Zoff/(1-Zdiag))/2;
R=sqrt(1+((1-Zdiag)/(Zdiag-(1+Zdiag)*(cos(theta))^2)));
theta_deg=theta*180/pi;