%	ANISO_SG_GRID Elliptic anisotropy parameters R and theta of gridded data.
%   [R, THETA_DEG]=ANISO_SG_GRID(XI,YI,ZI) returns the anisotropy ratio (R)
%   and the principal axes orientation angle (THETA_DEG) for a 2D random
%   field ZI measured on a recntagular grid [XI,YI]
%
%   The sample partial derivatives are evaluated using the method of
%   Savitzky Golay smoothing polynomials of order P and window size L. 
%
%   THETA_DEG represents the angle between the x axis of the coordinate
%   system and the first (not necessarily the major) principal axis
%   R represents the ratio of the correlation length along the first
%   principal axis over the correlation lengh along the orthogonal 
%   principal axis. 
%
%   INPUT VARIABLES
%
%   XI:	Grid coordinates on X-axis
%   YI:	Grid coordinates on Y-axis
%   ZI:	Random Field on grid [XI,YI]
%	P:  Savitzky Golay filter polynomial order
%	L:	Savitzky-Golay window size
%
%	OUTPUT VARIABLES
%
%	R:          Anisotropy ratio
%	THETA_DEG:	Anisotropy orientiation angle (degrees)
%
%   COMMENTS:
%
%   The anisotropy ratio is defined as R = b1/b2, where b1 is the 
%   correlation length in the direction 1 (horizontal) and b2 is the 
%   correlation length in the direction 2 (vertical)  
%
%   EXAMPLE:
%
%   Generate and display a 2D random field with Matern covariance and
%   smoothness index NU=2, and then determine the anisotropy parameters
%   from the sample using ANISO_SG_GRID with polynomial filter of order P=5
%   and window size W=7
%
%   [x,y,rf]=rf_gen(100,100,'Mate',0,0.5,4,6,20,2);
%   [R, THETA_DEG]=aniso_sg_grid(x,y,rf,5,7)
%
%   Code written by D.T. Hristopulos and A. Chorti
%   Last update: August 15, 2008


function [R, theta_deg]=aniso_sg_grid(xi,yi,zi,p,l)
stepx=xi(1, 2)-xi(1, 1);
stepy=yi(2, 1)-yi(1, 1);
[B, D]=sgolay(p, l);
grad=D(:, 2)';

for m=1:length(xi)
    for k=(l-1)/2+1:length(yi)-(l-1)/2
        divZx_sg(m, k-(l-1)/2)=(1/stepx*grad*zi(m, k-(l-1)/2:k+(l-1)/2)');
    end
end
for m=1:length(yi)
    for k=(l-1)/2+1:length(xi)-(l-1)/2
        divZy_sg(k-(l-1)/2, m)=(1/stepy*grad*zi(k-(l-1)/2:k+(l-1)/2, m));
    end
end

divZx_sg=divZx_sg((l-1)/2+1:end-(l-1)/2, :);
divZy_sg=divZy_sg(:, (l-1)/2+1:end-(l-1)/2);
Q11sg=mean(mean(divZx_sg.*divZx_sg, 1), 2);
Q22sg=mean(mean(divZy_sg.*divZy_sg, 1), 2);
Q12sg=mean(mean(divZx_sg.*divZy_sg, 1), 2);
Zdiag=Q22sg/Q11sg;
Zoff=Q12sg/Q11sg;
theta= atan2(2*Zoff,1-Zdiag)/2;
R=sqrt(1+((1-Zdiag)/(Zdiag-(1+Zdiag)*(cos(theta))^2)));
theta_deg=theta*180/pi;