%	RF_GEN Normal, correlated 2D random fields on a grid
%	[X,Y,RF]=RF_GEN(LX,LY,COR,FM,SF,XI1,XI2,THETA,NU) returns the 2D random
%	field RF on the input grid [X,Y].
%
%	The field is generated by means of the fast spectral method
%	(Fourier Filtering Method).
%	This method uses efficiently the FFT2. 
%
%	INPUT VARIABLES:
%
%	LX:		Length of the domain in the x-direction (integer)
%	LY:		Length of the domain in the y-direction (integer)
%	FM:		Mean of RF
%	SF:		Standard deviation of RF
%	COR:	Correlation model (string)
%	xi1:    Correlation length along x
%	xi2:	Correlation length along y
%	THETA:	Angle of the first principal direction with respect to the x
%           axis (degrees)
%   nu:     The Matern smoothness degree (if not supplied it is set
%           arbitrarily to NU=5). It is only used for the Matern
%           correlation.
%
%	OUTPUT VARIABLES:
%
%	[X,Y]:	Numerical rectangular grid [0, LX] x [0, LY]
%	RF:		Random field values on the grid
%
%	COMMENTS:
%
%	Three correlation models are currently supported:
%
%	'Gaus':	Gaussian:		exp(-r^2)
%	'Expo':	Exponential:	exp[-|r|]
%	'Mate':	Matern:	c0*2^(1-nu)*Gamma(nu)*sqrt(nu)*r^nu*K(nu, r)
%
%   EXAMPLE:
%
%   To generate and display a 2D random field with Matern covariance and
%   smoothness index NU=2 use the following:
%
%   [x,y,rf]=rf_gen(100,100,'Mate',0,0.5,4,6,20,2);
%   surf(x,y,rf); shading flat;
%
%   Code written by D.T. Hristopulos and A. Chorti
%   Last update: August 15, 2008

function [x,y,rf]=rf_gen(Lx,Ly,cor,fm,sf,xi1,xi2,theta,nu)

if nargin==8
    nu=5;
end
nx=1:Lx;
ny=1:Ly;  
knx=2*pi*(nx/Lx-1/2);
kny=2*pi*(ny/Ly-1/2);
[x,y]=meshgrid(nx,ny);
[kxo,kyo]=meshgrid(knx,kny);
th=theta*pi/180;
A=(xi1*cos(th))^2+(xi2*sin(th))^2;
B=(xi2*cos(th))^2+(xi1*sin(th))^2;
C=(2*sin(th)*cos(th)*(xi1^2-xi2^2));

if  strcmp(cor,'Gaus')
	tit='Gaussian correlation';
	Sq=pi*xi1*xi2*exp(-1/4*(A*kxo.^2+B*kyo.^2+C*kxo.*kyo));
elseif strcmp(cor,'Mate')
	tit='Matern correlation';
    c0=1;
    a=1;
	Sq=c0*1./((a^2+A*kxo.^2+B*kyo.^2+C*kxo.*kyo).^(nu+1));
elseif	strcmp(cor,'Expo')
	tit='Exponential correlation';
	Sq=(2*pi*xi1*xi2./(sqrt(1^2+A*kxo.^2+B*kyo.^2+C*kxo.*kyo).^3));
else
	sprintf('%s','Unrecognizable Model')
	pause;
end;
Sq=[Sq(Lx/2/1+1:end, Ly/2/1+1:end), Sq(Lx/2/1+1:end, 1:Ly/2/1); 
    Sq(1:Lx/2/1, Ly/2/1+1:end), Sq(1:Lx/2/1, 1:Ly/2/1)];

u=zeros(Ly,Lx);
nlx=Lx/2+1;
nly=Ly/2+1;
p1=2*pi*rand(nly,nlx);
u1=exp(-1i*p1);
u1(1,1)=real(u1(1,1));
u1(1,nlx)=real(u1(1,nlx));
u1(nly,1)=real(u1(nly,1));
u1(nly,nlx)=real(u1(nly,nlx));
u(1:nly,1:nlx)=u1;
for i=nlx+1:Lx
    u(1,i)=conj(u1(1,Lx+2-i));
    u(nly,i)=conj(u1(nly,Lx+2-i));
end;

for i=nly+1:Ly
    u(i,1)=conj(u1(Ly+2-i,1));
    u(i,nlx)=conj(u1(Ly+2-i,nlx));
end;

u1part=u1(2:nly-1,2:nlx-1);
p2=2*pi*rand(nly-2,nlx-2);
u2=exp(-1i*p2);
u(nly+1:Ly,nlx+1:Lx)=conj(rot90(u1part,2));
u(nly+1:Ly,2:nlx-1)=u2;
u(2:nly-1,nlx+1:Lx)=conj(rot90(u2,2));
Ssq=sqrt(Sq).*u;
rf1=sf*sqrt(Lx*Ly)*real(ifft2(Ssq));
tempstd=std(reshape(rf1, [(Ly)*(Lx) 1]));
rf1=rf1/(tempstd/sf);
rf=fm+rf1-mean(mean(rf1,1));
%srf=std(reshape(rf, [(Ly)*(Lx) 1]));


